package main

import (
	"bytes"
	"os"
	"os/exec"
	"text/template"

	"codeberg.org/gruf/go-errors/v2"
)

func main() {
	// Load main bit flags template file
	flagTpl, err := template.ParseFiles("flag.tpl")
	if err != nil {
		panic(err)
	}

	// Load bit flags tests template file
	tflagTpl, err := template.ParseFiles("flag_test.tpl")
	if err != nil {
		panic(err)
	}

	flags := []struct {
		Size int
		Bits []int
	}{
		{Size: 8, Bits: bits(8)},
		{Size: 16, Bits: bits(16)},
		{Size: 32, Bits: bits(32)},
		{Size: 64, Bits: bits(64)},
	}

	// Generate bit flags Go file
	if err := generate(flagTpl, "flag.go", flags); err != nil {
		panic(err)
	}

	// Generate bit flags Go tests file
	if err := generate(tflagTpl, "flag_test.go", flags); err != nil {
		panic(err)
	}
}

func generate(tpl *template.Template, filename string, data interface{}) error {
	var buf bytes.Buffer

	// Execute the template for atomics type
	if err := tpl.Execute(&buf, data); err != nil {
		return errors.Wrap(err, "error executing template for '"+filename+"'")
	}

	// Open file for the atomic type
	file, err := os.OpenFile(filename, os.O_CREATE|os.O_TRUNC|os.O_WRONLY, 0o644)
	if err != nil {
		return errors.Wrap(err, "error opening file '"+filename+"'")
	}

	// Write generated buffer contents to file
	if _, err := buf.WriteTo(file); err != nil {
		return errors.Wrap(err, "error writing to file '"+filename+"'")
	} else if err := file.Close(); err != nil {
		return errors.Wrap(err, "error closing file '"+filename+"'")
	}

	// Run gofmt on generated file
	cmd := exec.Command("gofumports", "-w", filename)
	if err := cmd.Run(); err != nil {
		return errors.Wrap(err, "error running gofmt -w "+filename)
	}

	return nil
}

func bits(count int) []int {
	var bits []int
	for i := 0; i < count; i++ {
		bits = append(bits, i)
	}
	return bits
}
