package bitutil_test

import (
	"testing"

	"codeberg.org/gruf/go-bitutil"
)

func TestPackInt8s(t *testing.T) {
	for i1 := int8(-100); i1 < 100; i1 += 10 {
		for i2 := int8(0); i2 < 100; i2 += 10 {
			pack := bitutil.PackInt8s(i1, i2)
			if pack != (uint16(i1)<<8 | uint16(i2)&((1<<8)-1)) {
				t.Error("failed packing int8s")
			}
		}
	}
}

func TestUnpackInt8s(t *testing.T) {
	for i1 := int8(-100); i1 < 100; i1 += 10 {
		for i2 := int8(0); i2 < 100; i2 += 10 {
			pack := uint16(i1)<<8 | uint16(i2)&((1<<8)-1)
			if t1, t2 := bitutil.UnpackInt8s(pack); t1 != i1 || t2 != i2 {
				t.Error("failed unpacking int8s")
			}
		}
	}
}

func TestPackInt16s(t *testing.T) {
	for i1 := int16(-10000); i1 < 10000; i1 += 10 {
		for i2 := int16(-10000); i2 < 10000; i2 += 10 {
			pack := bitutil.PackInt16s(i1, i2)
			if pack != (uint32(i1)<<16 | uint32(i2)&((1<<16)-1)) {
				t.Error("failed packing int16s")
			}
		}
	}
}

func TestUnpackInt16s(t *testing.T) {
	for i1 := int16(-10000); i1 < 10000; i1 += 10 {
		for i2 := int16(-10000); i2 < 10000; i2 += 10 {
			pack := uint32(i1)<<16 | uint32(i2)&((1<<16)-1)
			if t1, t2 := bitutil.UnpackInt16s(pack); t1 != i1 || t2 != i2 {
				t.Error("failed unpacking int16s")
			}
		}
	}
}

func TestPackInt32s(t *testing.T) {
	for i1 := int32(-100000); i1 < 100000; i1 += 10 {
		for i2 := int32(-100000); i2 < 100000; i2 += 10 {
			pack := bitutil.PackInt32s(i1, i2)
			if pack != (uint64(i1)<<32 | uint64(i2)&((1<<32)-1)) {
				t.Error("failed packing int32s")
			}
		}
	}
}

func TestUnpackInt32s(t *testing.T) {
	for i1 := int32(-100000); i1 < 100000; i1 += 10 {
		for i2 := int32(-100000); i2 < 100000; i2 += 10 {
			pack := uint64(i1)<<32 | uint64(i2)&((1<<32)-1)
			if t1, t2 := bitutil.UnpackInt32s(pack); t1 != i1 || t2 != i2 {
				t.Error("failed unpacking int32s")
			}
		}
	}
}

func TestPackUint8s(t *testing.T) {
	for u1 := uint8(0); u1 < 200; u1 += 10 {
		for u2 := uint8(0); u2 < 200; u2 += 10 {
			pack := bitutil.PackUint8s(u1, u2)
			if pack != (uint16(u1)<<8 | uint16(u2)&((1<<8)-1)) {
				t.Error("failed packing uint8s")
			}
		}
	}
}

func TestUnpackUint8s(t *testing.T) {
	for u1 := uint8(0); u1 < 200; u1 += 10 {
		for u2 := uint8(0); u2 < 200; u2 += 10 {
			pack := uint16(u1)<<8 | uint16(u2)&((1<<8)-1)
			if t1, t2 := bitutil.UnpackUint8s(pack); t1 != u1 || t2 != u2 {
				t.Error("failed unpacking uint8s")
			}
		}
	}
}

func TestPackUint16s(t *testing.T) {
	for u1 := uint16(0); u1 < 20000; u1 += 10 {
		for u2 := uint16(0); u2 < 20000; u2 += 10 {
			pack := bitutil.PackUint16s(u1, u2)
			if pack != (uint32(u1)<<16 | uint32(u2)&((1<<16)-1)) {
				t.Error("failed packing uint16s")
			}
		}
	}
}

func TestUnpackUint16s(t *testing.T) {
	for u1 := uint16(0); u1 < 20000; u1 += 10 {
		for u2 := uint16(0); u2 < 20000; u2 += 10 {
			pack := uint32(u1)<<16 | uint32(u2)&((1<<16)-1)
			if t1, t2 := bitutil.UnpackUint16s(pack); t1 != u1 || t2 != u2 {
				t.Error("failed unpacking uint16s")
			}
		}
	}
}

func TestPackUint32s(t *testing.T) {
	for u1 := uint32(0); u1 < 200000; u1 += 10 {
		for u2 := uint32(0); u2 < 200000; u2 += 10 {
			pack := bitutil.PackUint32s(u1, u2)
			if pack != (uint64(u1)<<32 | uint64(u2)&((1<<32)-1)) {
				t.Error("failed packing uint32s")
			}
		}
	}
}

func TestUnpackUint32s(t *testing.T) {
	for u1 := uint32(0); u1 < 200000; u1 += 10 {
		for u2 := uint32(0); u2 < 200000; u2 += 10 {
			pack := uint64(u1)<<32 | uint64(u2)&((1<<32)-1)
			if t1, t2 := bitutil.UnpackUint32s(pack); t1 != u1 || t2 != u2 {
				t.Error("failed unpacking uint32s")
			}
		}
	}
}
